module mgetbase
   use precision
   use constants
   use mfunctions, only: sdot
   implicit none

   private

   public :: getbase

contains

      subroutine getbase(p1,p2,r1,r2,e1,e2,e3,e4)
      implicit none

      integer j

      real(ki), dimension(4), intent(in) :: p1, p2
      real(ki), dimension(4), intent(out) :: e1, e2

      real(ki) :: gamma, MP12, MP11, MP22, den, se1p, se2p, se1m, se2m
      real(ki), intent(out) :: r1, r2
      complex(ki) :: f1, f2, phie1p, phie1m, phie2p, phie2m
      complex(ki), intent(out) :: e3(4), e4(4)

      MP11=sdot(p1,p1)
      MP12=sdot(p1,p2)
      MP22=sdot(p2,p2)

      gamma=MP12+sign(+1.0_ki,MP12)*sqrt(MP12**2-MP11*MP22)

!--- Warning: if p1 ~ p2 and massless this formula brakes down
      if ( abs(gamma) .lt. zip1 ) then
       write(6,*) 'getbase: small gamma'
       write(6,*) 'gamma=', gamma
       write(6,*) ' MP11=', MP11
       write(6,*) ' MP12=', MP12
       write(6,*) ' MP22=', MP22
      endif

      r1=MP11/gamma
      r2=MP22/gamma
      den=1.0_ki-r1*r2

         do j=1,4
            e1(j)=(p1(j)-r1*p2(j))/den
            e2(j)=(p2(j)-r2*p1(j))/den
         enddo

!-----positive energy case
         if (e1(4) .gt. 0.0_ki) then
            se1p=sqrt(e1(4)+e1(1))
            se1m=sqrt(e1(4)-e1(1))
            phie1p=cmplx(e1(2),+e1(3),ki)/se1p/se1m
            phie1m=cmplx(e1(2),-e1(3),ki)/se1p/se1m
            f1=cone
         else
!-----negative energy case
            se1p=sqrt(-e1(4)-e1(1))
            se1m=sqrt(-e1(4)+e1(1))
            phie1p=-cmplx(e1(2),+e1(3),ki)/se1p/se1m
            phie1m=-cmplx(e1(2),-e1(3),ki)/se1p/se1m
            f1=im
         endif

!-----positive energy case
         if (e2(4) .gt. 0.0_ki) then
            se2p=sqrt(e2(4)+e2(1))
            se2m=sqrt(e2(4)-e2(1))
            phie2p=cmplx(e2(2),+e2(3),ki)/se2p/se2m
            phie2m=cmplx(e2(2),-e2(3),ki)/se2p/se2m
            f2=cone
         else
!-----negative energy case
            se2p=sqrt(-e2(4)-e2(1))
            se2m=sqrt(-e2(4)+e2(1))
            phie2p=-cmplx(e2(2),+e2(3),ki)/se2p/se2m
            phie2m=-cmplx(e2(2),-e2(3),ki)/se2p/se2m
            f2=im
         endif

         e3(4)=f1*f2*(+phie1p*phie2m*se1m*se2m+se1p*se2p)/2.0_ki
         e3(1)=f1*f2*(-phie1p*phie2m*se1m*se2m+se1p*se2p)/2.0_ki
         e3(2)=f1*f2*(+phie2m*se1p*se2m+phie1p*se1m*se2p)/2.0_ki
         e3(3)=f1*f2*(+phie2m*se1p*se2m-phie1p*se1m*se2p)/2.0_ki*im

         e4(4)=f1*f2*(+phie2p*phie1m*se2m*se1m+se2p*se1p)/2.0_ki
         e4(1)=f1*f2*(-phie2p*phie1m*se2m*se1m+se2p*se1p)/2.0_ki
         e4(2)=f1*f2*(+phie1m*se2p*se1m+phie2p*se2m*se1p)/2.0_ki
         e4(3)=f1*f2*(+phie1m*se2p*se1m-phie2p*se2m*se1p)/2.0_ki*im

  end subroutine getbase
end module mgetbase

